"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
exports.__esModule = true;
var commander = require("commander");
var common_1 = require("../common");
var config_1 = require("../config");
var changeCase = require("change-case");
var CONFIG_DATA_FIELD_NAMES = common_1.getFieldNamesInType({ email: 0, password: 0, downloadPath: 0 });
function hasAnyValues(options) {
    return CONFIG_DATA_FIELD_NAMES.reduce(function (result, fieldName) {
        return result || options[fieldName] != undefined;
    }, false);
}
function displaySettings(options) {
    var fieldsToDisplay = CONFIG_DATA_FIELD_NAMES
        .map(function (fieldName) { return ({
        friendlyName: changeCase.title(fieldName),
        value: fieldName == 'password' ? (options[fieldName] && '********') : options[fieldName]
    }); });
    console.log();
    console.log('Settings:');
    console.log();
    for (var _i = 0, fieldsToDisplay_1 = fieldsToDisplay; _i < fieldsToDisplay_1.length; _i++) {
        var field = fieldsToDisplay_1[_i];
        console.log("  " + field.friendlyName + ": " + field.value);
    }
    console.log();
}
function getUpdatedSettings(existingOptions, newOptions) {
    var result = __assign({}, existingOptions);
    for (var _i = 0, CONFIG_DATA_FIELD_NAMES_1 = CONFIG_DATA_FIELD_NAMES; _i < CONFIG_DATA_FIELD_NAMES_1.length; _i++) {
        var fieldName = CONFIG_DATA_FIELD_NAMES_1[_i];
        var newValue = newOptions[fieldName];
        if (newValue != undefined) {
            result[fieldName] = newValue;
        }
    }
    return result;
}
function updateSettings(config, options) {
    config.data = getUpdatedSettings(config.data, options);
    return config.save().then(function () {
        console.log();
        console.log('Settings updated');
        console.log();
    });
}
commander
    .command('config')
    .description('Configures settings')
    .option('-e, --email <email>')
    .option('-p, --password <password>')
    .option('-d, --download-path <download-path>')
    .action(function (options) {
    config_1.Config.load().then(function (config) {
        if (!hasAnyValues(options)) {
            return displaySettings(config.data);
        }
        return updateSettings(config, options);
    });
});
//# sourceMappingURL=config.js.map