"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
exports.__esModule = true;
var path = require("path");
var files_1 = require("../utils/files");
var commander = require("commander");
var egghead_1 = require("../egghead");
var chalk = require("chalk");
var common_1 = require("../common");
var config_1 = require("../config");
var linq = require("linq");
var http_1 = require("../utils/http");
var Promise = require("bluebird");
var terminal = require('terminal-kit').terminal;
var formatFileSize = require("filesize");
var CircularJson = require('circular-json');
var pad = require("pad");
var sanitizeFileName = require("sanitize-filename");
var NO_OP = function () { };
var NO_CONCURRENCY = { concurrency: 1 };
commander
    .command('download')
    .option('-t, --technology <technology-name>')
    .option('-c, --course <course-name>')
    .option('-o, --overwrite')
    .option('-n, --no-verify')
    .description('Downloads course(s)')
    .action(function (cliOptions) {
    var options = parseOptions(cliOptions);
    handleDownloadCommand(options);
});
var TechnologyNotFound = (function (_super) {
    __extends(TechnologyNotFound, _super);
    function TechnologyNotFound(filterName) {
        return _super.call(this, "No technology section found named \"" + filterName + "\"") || this;
    }
    return TechnologyNotFound;
}(Error));
exports.TechnologyNotFound = TechnologyNotFound;
var CourseNotFound = (function (_super) {
    __extends(CourseNotFound, _super);
    function CourseNotFound(filterName) {
        return _super.call(this, "No course found named \"" + filterName + "\"") || this;
    }
    return CourseNotFound;
}(Error));
exports.CourseNotFound = CourseNotFound;
var FilterNotSpecified = (function (_super) {
    __extends(FilterNotSpecified, _super);
    function FilterNotSpecified() {
        return _super.call(this, 'A technology or course name must be specified') || this;
    }
    return FilterNotSpecified;
}(Error));
exports.FilterNotSpecified = FilterNotSpecified;
function handleDownloadCommand(options) {
    return config_1.Config.load().then(function (config) {
        var filter = options.filter;
        if (!config.data.email || !config.data.password) {
            return common_1.displayCredentialsNotSet();
        }
        if (!config.data.downloadPath) {
            return common_1.displayDownloadPathNotSet();
        }
        var egghead = new egghead_1.EggHead();
        return common_1.authenticate(egghead, config.data.email, config.data.password)
            .then(function () { return common_1.getCourses(egghead); })
            .then(function (technologies) {
            if (!filter) {
                return technologies;
            }
            technologies = filterTechnologiesAndCourses(filter, technologies);
            return checkFoundMatchingCourses(filter, technologies);
        })
            .then(function (technologies) { return getLessons(egghead, technologies); })
            .then(function (technologies) {
            displayMatchingCourses(technologies);
            var confirm = (options.noVerify ? Promise.resolve(true) : confirmDownloadCourses(technologies));
            return confirm.then(function (confirmed) {
                if (!confirmed) {
                    return process.exit(0);
                }
                return downloadTechnologies(egghead, technologies, config.data.downloadPath);
            });
        })
            .then(function () {
            console.log(chalk.green('✔ Downloads Finished Successfully'));
            console.log();
        })
            .then(NO_OP);
    })
        .then(function () { return process.exit(0); })["catch"](common_1.handleErrorAndExit);
}
function parseOptions(options) {
    var result = {
        filter: options.technology ? { type: 'technology', name: options.technology } :
            options.course ? { type: 'course', name: options.course } :
                null,
        overwrite: options.overwrite,
        noVerify: options.noVerify
    };
    return result;
}
function filterTechnologiesAndCourses(filter, technologies) {
    if (filter.type === 'technology') {
        return linq
            .from(technologies)
            .where(function (technology) { return technology.name.toLowerCase().indexOf(filter.name.toLowerCase()) > -1; })
            .toArray();
    }
    else {
        return linq
            .from(technologies)
            .select(function (technology) {
            return __assign({}, technology, { courses: filterCourses(filter, technology.courses) });
        })
            .toArray();
    }
}
function filterCourses(filter, courses) {
    return linq
        .from(courses)
        .where(function (course) { return course.name.toLowerCase().indexOf(filter.name.toLowerCase()) > -1; })
        .toArray();
}
function checkFoundMatchingCourses(filter, technologies) {
    if (technologies.length == 0) {
        if (filter.type === 'technology') {
            throw new TechnologyNotFound(filter.name);
        }
        else {
            throw new CourseNotFound(filter.name);
        }
    }
    return technologies;
}
function selectCourses(technologies) {
    return linq
        .from(technologies)
        .selectMany(function (technology) { return technology.courses; })
        .toArray();
}
function countLessons(technologies) {
    return linq
        .from(technologies)
        .sum(function (technology) { return linq
        .from(technology.courses)
        .sum(function (course) { return course.lessonCount; }); });
}
function displayMatchingCourses(technologies) {
    var courseCount = selectCourses(technologies).length;
    console.log("Found " + courseCount + " matching courses");
    console.log();
    var totalDownloadSize = 0;
    for (var _i = 0, technologies_1 = technologies; _i < technologies_1.length; _i++) {
        var technology = technologies_1[_i];
        var technologyFileSize = linq
            .from(technology.courses)
            .selectMany(function (course) { return course.lessons; })
            .sum(function (lesson) { return lesson.fileSize; });
        totalDownloadSize += technologyFileSize;
        console.log("" + technology.name + chalk.yellow(" (" + technology.courses.length + " courses, " + formatFileSize(technologyFileSize) + ")"));
        console.log();
        for (var _a = 0, _b = technology.courses; _a < _b.length; _a++) {
            var course = _b[_a];
            var courseFileSize = linq
                .from(course.lessons)
                .sum(function (lesson) { return lesson.fileSize; });
            console.log(" \u2022 " + course.name + chalk.yellow(" (" + course.lessonCount + " lessons, " + formatFileSize(courseFileSize) + ")"));
        }
    }
    console.log();
    console.log("Total Download Size: " + chalk.yellow(formatFileSize(totalDownloadSize)));
}
function confirmDownloadCourses(technologies) {
    return new Promise(function (resolve, reject) {
        var courseCount = selectCourses(technologies).length;
        console.log();
        console.log('Download ' + (courseCount == 1 ? 'this course' : "these " + courseCount + " courses? [y|n]"));
        console.log();
        terminal.yesOrNo({ yes: ['y', 'Y'], no: ['n', 'N'] }, function (err, result) {
            return err ? reject(err) : resolve(result);
        });
    });
}
function getLessons(egghead, technologies) {
    var lessonCount = countLessons(technologies);
    var checkedLessons = 0;
    var spinner = common_1.createSpinner("Retrieving lesson information (0/" + lessonCount + ")...");
    var MAX_CONCURRENT_FILE_SIZE_REQUESTS = 10;
    return Promise.map(technologies, function (technology) {
        return Promise.map(technology.courses, function (course) {
            return getCourseLessons(egghead, course).then(function (course) {
                return Promise
                    .map(course.lessons, function (lesson) {
                    return getLessonFileSize(lesson).then(function (lesson) {
                        checkedLessons++;
                        spinner.text = "Retrieving lesson information (" + checkedLessons + "/" + lessonCount + ")...";
                        return lesson;
                    });
                }, { concurrency: MAX_CONCURRENT_FILE_SIZE_REQUESTS })
                    .then(function (lessonsWithFileSizes) {
                    var result = __assign({}, course, { lessons: lessonsWithFileSizes });
                    return result;
                });
            });
        }, NO_CONCURRENCY).then(function (courses) {
            return __assign({}, technology, { courses: courses });
        });
    }, NO_CONCURRENCY).then(function (result) {
        spinner.succeed("Retrieved lesson information (" + checkedLessons + "/" + lessonCount + ")");
        console.log();
        return result;
    }, function (err) {
        spinner.fail();
        throw err;
    });
}
function getCourseLessons(egghead, course) {
    return egghead
        .getCourseLessons(course.code)
        .then(function (lessons) { return Promise.map(lessons, getLessonFileSize, NO_CONCURRENCY); })
        .then(function (lessons) {
        var result = __assign({}, course, { lessons: lessons });
        return result;
    });
}
function getLessonFileSize(lesson) {
    return http_1.getFileSize(lesson.url).then(function (fileSize) {
        return __assign({}, lesson, { fileSize: fileSize });
    });
}
function downloadTechnologies(egghead, technologies, downloadPath) {
    var lessons = linq
        .from(technologies)
        .selectMany(function (technology) { return technology.courses.map(function (course) { return ({ technology: technology, course: course }); }); })
        .selectMany(function (item) { return item.course.lessons.map(function (lesson) { return (__assign({}, item, { lesson: lesson })); }); })
        .orderBy(function (item) { return item.technology.name; })
        .thenBy(function (item) { return item.course.name; })
        .thenBy(function (item) { return item.lesson.lessonNumber; })
        .toArray();
    return Promise
        .mapSeries(lessons, function (_a, index) {
        var technology = _a.technology, course = _a.course, lesson = _a.lesson;
        return downloadLesson(egghead, technology, course, lesson, downloadPath, index, lessons.length);
    })
        .then(NO_OP);
}
function checkFileAlreadyDownloaded(fileName, expectedFileSize) {
    return files_1.getFileSize(fileName).then(function (fileSize) {
        if (expectedFileSize != fileSize) {
            return files_1.deleteFile(fileName).then(function () { return false; });
        }
        else {
            return true;
        }
    }, function () {
        return false;
    });
}
function downloadLesson(egghead, technology, course, lesson, downloadPath, downloadIndex, downloadCount) {
    var technologyPath = getTechnologyPath(downloadPath, technology);
    var coursePath = getCoursePath(technologyPath, course);
    var lessonPath = getLessonFileName(coursePath, lesson);
    return files_1.ensureDirectoryExists(coursePath).then(function () {
        var lessonFileName = getLessonFileName(coursePath, lesson);
        return checkFileAlreadyDownloaded(lessonFileName, lesson.fileSize).then(function (alreadyDownloaded) {
            if (alreadyDownloaded) {
                return;
            }
            var UPDATE_INTERVAL = 250;
            var downloaded = 0;
            var total = 0;
            function getStatusText(verb) {
                return verb + " " + (downloadIndex + 1) + "/" + downloadCount + ": " + lesson.name + " (" + formatFileSize(downloaded) + "/" + formatFileSize(lesson.fileSize) + ")";
            }
            var spinner = common_1.createSpinner(getStatusText('Downloading'));
            var interval = setInterval(function () {
                spinner.text = getStatusText('Downloading');
            }, UPDATE_INTERVAL);
            var progressOptions = {
                update: function (newDownloaded, newTotal) {
                    downloaded = newDownloaded;
                    total = newTotal;
                }
            };
            return egghead
                .downloadFile(lesson.url, lessonFileName, progressOptions)
                .then(function () {
                clearInterval(interval);
                spinner.succeed(getStatusText('Downloaded'));
                console.log();
            }, function (err) {
                clearInterval(interval);
                spinner.fail(getStatusText('Failed'));
                console.log();
                throw err;
            });
        });
    });
}
function getTechnologyPath(downloadPath, technology) {
    return path.join(downloadPath, technology.name);
}
function getCoursePath(technologyPath, course) {
    return path.join(technologyPath, course.name);
}
function getLessonFileName(coursePath, lesson) {
    var lessonNumber = pad(2, lesson.lessonNumber.toString(), '0');
    var fileTitle = sanitizeFileName(lessonNumber + " " + lesson.name, { replacement: ' ' }).replace(/\s+/g, ' ');
    return path.join(coursePath, fileTitle + ".mp4");
}
//# sourceMappingURL=download.js.map