"use strict";
exports.__esModule = true;
var commander = require("commander");
var egghead_1 = require("../egghead");
var chalk = require("chalk");
var common_1 = require("../common");
var config_1 = require("../config");
var pkg = require('../../package.json');
commander
    .command('list')
    .description('Lists all courses')
    .action(function () {
    console.log();
    config_1.Config.load().then(function (config) {
        if (!config.data.email || !config.data.password) {
            return common_1.displayCredentialsNotSet();
        }
        var egghead = new egghead_1.EggHead();
        return common_1.authenticate(egghead, config.data.email, config.data.password)
            .then(function () { return common_1.getCourses(egghead); })
            .then(function (technologies) { return listCourses(technologies); });
    });
});
function listCourses(technologies) {
    var allCourses = technologies
        .map(function (technology) { return technology.courses; })
        .reduce(function (result, courses) { return result.concat(courses); }, []);
    var totalLessonCount = allCourses
        .map(function (course) { return course.lessonCount; })
        .reduce(function (total, count) { return total + count; }, 0);
    console.log('Course Listing: ' + chalk.yellow(totalLessonCount + " lessons over " + allCourses.length + " courses over " + technologies.length + " technologies"));
    console.log();
    for (var _i = 0, technologies_1 = technologies; _i < technologies_1.length; _i++) {
        var technology = technologies_1[_i];
        console.log(chalk.bold(technology.name) + chalk.yellow(" (" + technology.courses.length + " courses)"));
        console.log();
        for (var _a = 0, _b = technology.courses; _a < _b.length; _a++) {
            var course = _b[_a];
            console.log(" \u2022 " + course.name + chalk.yellow(" (" + course.lessonCount + " lessons)"));
        }
        console.log();
    }
}
//# sourceMappingURL=list.js.map